@@ -138,6 +138,15 @@ vinfo_t* PsycoObject_GetItem(PsycoObject* po, vinfo_t* o, vinfo_t* key)
 			vinfo_decref(key_value, po);
 			return result;
 		}
+		if (TYPE_HAS_INDEX(ktp)) {
+			vinfo_t* result;
+			vinfo_t* key_value = TYPE_VINDEX(po, ktp, key);
+			if (key_value == NULL)
+				return NULL;
+			result = PsycoSequence_GetItem(po, o, key_value);
+			vinfo_decref(key_value, po);
+			return result;
+		}
 		type_error(po, "sequence index must be integer");
 		return false;
 	}
@@ -183,6 +192,15 @@ bool PsycoObject_SetItem(PsycoObject* po, vinfo_t* o, vinfo_t* key,
 			vinfo_decref(key_value, po);
 			return result;
 		}
+		if (TYPE_HAS_INDEX(ktp)) {
+			bool result;
+			vinfo_t* key_value = TYPE_VINDEX(po, ktp, key);
+			if (key_value == NULL)
+				return false;
+			result = PsycoSequence_SetItem(po, o, key_value,value);
+			vinfo_decref(key_value, po);
+			return result;
+		}
 		if (tp->tp_as_sequence->sq_ass_item) {
 			type_error(po, "sequence index must be integer");
 			return false;
@@ -692,7 +710,7 @@ vinfo_t* PsycoNumber_Add(PsycoObject* po, vinfo_t* v, vinfo_t* w)
 	return result;
 }
 
-static vinfo_t* psequence_repeat(PsycoObject* po, intargfunc repeatfunc,
+static vinfo_t* psequence_repeat(PsycoObject* po, void *repeatfunc,
 				 vinfo_t* vseq, vinfo_t* vn)
 {
 	PyTypeObject* tp = Psyco_FastType(vn);
@@ -705,6 +723,9 @@ static vinfo_t* psequence_repeat(PsycoObject* po, intargfunc repeatfunc,
 	else if (PyType_TypeCheck(tp, &PyLong_Type)) {
 		vcount = PsycoLong_AsLong(po, vn);
 	}
+	else if (TYPE_HAS_INDEX(tp)) {
+		vcount = TYPE_VINDEX(po, tp, vn);
+	}
 	else {
 		return type_error(po,
 			"can't multiply sequence to non-int");
@@ -779,7 +800,35 @@ vinfo_t* PsycoNumber_Remainder(PsycoObject* po, vinfo_t* v, vinfo_t* w)
 DEFINEFN
 vinfo_t* PsycoNumber_Power(PsycoObject* po, vinfo_t* v1, vinfo_t* v2, vinfo_t*v3)
 {
-	/* XXX implement the ternary operators */
+	/* limited support -- the goal is only to support
+	   (int-or-long-or-float) ** (int-or-long-or-float) */
+	vinfo_t* x;
+	PyTypeObject* vtp;
+	PyTypeObject* wtp;
+	ternaryfunc f;
+	vtp = Psyco_NeedType(po, v1);
+	if (vtp == NULL)
+		return NULL;
+	if (vtp != &PyInt_Type && vtp != &PyLong_Type && vtp != &PyFloat_Type)
+		goto fallback;
+	wtp = Psyco_NeedType(po, v2);
+	if (wtp == NULL)
+		return NULL;
+	if (wtp != &PyInt_Type && wtp != &PyLong_Type && wtp != &PyFloat_Type)
+		goto fallback;
+	if (vtp == &PyFloat_Type || wtp == &PyFloat_Type)
+		f = PyFloat_Type.tp_as_number->nb_power;
+	else if (vtp == &PyLong_Type || wtp == &PyLong_Type)
+		f = PyLong_Type.tp_as_number->nb_power;
+	else
+		f = PyInt_Type.tp_as_number->nb_power;
+	x = Psyco_META3(po, f, CfReturnRef|CfPyErrIfNull,
+			"vvv", v1, v2, v3);
+	if (IS_IMPLEMENTED(x))
+		return x;  /* may be NULL */
+	vinfo_decref(x, po);
+
+ fallback:
 	return psyco_generic_call(po, PyNumber_Power, CfReturnRef|CfPyErrIfNull,
 				  "vvv", v1, v2, v3);
 }
@@ -876,7 +925,7 @@ vinfo_t* PsycoNumber_InPlaceMultiply(PsycoObject* po, vinfo_t* v, vinfo_t* w)
 	if (!IS_IMPLEMENTED(result)) {
 		PyTypeObject* vtp = Psyco_FastType(v);
 		PyTypeObject* wtp = Psyco_FastType(w);
-		intargfunc f = NULL;
+		void *f = NULL;
 		PySequenceMethods *mv = vtp->tp_as_sequence;
 		PySequenceMethods *mw = wtp->tp_as_sequence;
 		vinfo_decref(result, po);
