@@ -45,6 +45,8 @@ void Psyco_DefineMeta(void* c_function, void* psyco_function)
 			return;
 	}
 	if (c_function == NULL) {
+		/* known occurrences:
+		   in Python 2.1, PyString_Type.tp_richcompare == NULL */
 		debug_printf(1, ("init: C function pointer NULL in CPython\n"));
 		return;
 	}
@@ -315,6 +317,48 @@ vinfo_t* Psyco_Meta3x(PsycoObject* po, void* c_function, int flags,
 			(po, a1, a2, a3);
 }
 
+#if 0
+DEFINEFN
+int psyco_atcompiletime_mega(PsycoObject* po, vinfo_t* vi, long* out)
+{
+	if (!compute_vinfo(vi, po))
+		return -1;
+	if (is_runtime(vi->source)) {
+		if (vi->source & RunTime_Megamorphic)
+			return 0;
+		PycException_Promote(po, vi, &psyco_nonfixed_promotion_mega);
+		return -1;
+	}
+	else {
+		source_known_t* sk = CompileTime_Get(vi->source);
+		sk->refcount1_flags |= SkFlagFixed;
+		*out = sk->value;
+		return 1;
+	}
+}
+#endif
+
+DEFINEFN
+int psyco_pyobj_atcompiletime_mega(PsycoObject* po,
+				   vinfo_t* vi, PyObject** out)
+{
+	if (!compute_vinfo(vi, po))
+		return -1;
+	if (is_runtime(vi->source)) {
+		if (vi->source & RunTime_Megamorphic)
+			return 0;
+		PycException_Promote(po, vi,
+				     &psyco_nonfixed_pyobj_promotion_mega);
+		return -1;
+	}
+	else {
+		source_known_t* sk = CompileTime_Get(vi->source);
+		sk->refcount1_flags |= SkFlagFixed;
+		*out = (PyObject*) sk->value;
+		return 1;
+	}
+}
+
 
 /***************************************************************/
  /***   pyc_data_t                                            ***/
@@ -600,7 +644,7 @@ vinfo_t* PycException_Matches(PsycoObject* po, PyObject* e)
 	return result;
 }
 
-inline void clear_pseudo_exception(PsycoObject* po)
+PSY_INLINE void clear_pseudo_exception(PsycoObject* po)
 {
 	extra_assert(PycException_Occurred(po));
 	if (po->pr.tb != NULL) {
@@ -688,7 +732,7 @@ void cimpl_finalize_frame_locals(PyObject* f_exc_type,
 	PySys_SetObject("exc_traceback", f_exc_traceback);
 }
 
-inline void cimpl_set_exc_info(PyObject* target[],
+PSY_INLINE void cimpl_set_exc_info(PyObject* target[],
 			       PyObject** f_exc_type,
 			       PyObject** f_exc_value,
 			       PyObject** f_exc_traceback)
@@ -781,7 +825,7 @@ void PycException_Fetch(PsycoObject* po)
 	}
 }
 
-inline bool PycException_FetchNormalize(PsycoObject* po)
+PSY_INLINE bool PycException_FetchNormalize(PsycoObject* po)
 {
 	vinfo_t* result;
 	vinfo_array_t* array = array_new(3);
@@ -897,7 +941,7 @@ static PyObject* cimpl_vt_traceback(PyCodeObject* code, PyObject* globals,
 }
 
 /* record a traceback entry for the current virtual Python exception */
-inline void PsycoTraceBack_Here(PsycoObject* po, int lasti)
+PSY_INLINE void PsycoTraceBack_Here(PsycoObject* po, int lasti)
 {
 	int lineno = PyCode_Addr2Line(po->pr.co, lasti);
 	
@@ -1271,6 +1315,9 @@ static vinfo_t* _PsycoEval_SliceIndex(PsycoObject* po, vinfo_t* v)
 			result = vinfo_new(CompileTime_New(x));
 		}
 	}
+	else if (TYPE_HAS_INDEX(vtp)) {
+		result = TYPE_VINDEX(po, vtp, v);
+	}
 	else {
 		/* no error set */
 		result = NULL;
@@ -1819,13 +1866,13 @@ static void cimpl_do_raise(PyObject *type, PyObject *value, PyObject *tb)
 		Py_DECREF(tmp);
 	}
 
-	if (PyString_Check(type))
-		;
+	if (PyString_CheckExact(type))
+		/* warning skipped */;
 
-	else if (PyClass_Check(type))
+	else if (PyExceptionClass_Check(type))
 		PyErr_NormalizeException(&type, &value, &tb);
 
-	else if (PyInstance_Check(type)) {
+	else if (PyExceptionInstance_Check(type)) {
 		/* Raising an instance.  The value should be a dummy. */
 		if (value != Py_None) {
 			PyErr_SetString(PyExc_TypeError,
@@ -1836,7 +1883,7 @@ static void cimpl_do_raise(PyObject *type, PyObject *value, PyObject *tb)
 			/* Normalize to raise <class>, <instance> */
 			Py_DECREF(value);
 			value = type;
-			type = (PyObject*) ((PyInstanceObject*)type)->in_class;
+			type = PyExceptionInstance_Class(type);
 			Py_INCREF(type);
 		}
 	}
@@ -1844,8 +1891,9 @@ static void cimpl_do_raise(PyObject *type, PyObject *value, PyObject *tb)
 		/* Not something you can raise.  You get an exception
 		   anyway, just not what you specified :-) */
 		PyErr_Format(PyExc_TypeError,
-			     "exceptions must be strings, classes, or "
-			     "instances, not %s", type->ob_type->tp_name);
+			     "exceptions must be classes, instances, or "
+			     "strings (deprecated), not %s",
+			     type->ob_type->tp_name);
 		goto raise_error;
 	}
 	PyErr_Restore(type, value, tb);
@@ -1972,7 +2020,11 @@ cimpl_build_class(PyObject* g,  /* globals */
 }
 
 static PyObject* cimpl_import_name(PyObject* globals, PyObject* name,
-				   PyObject* fromlist)
+				   PyObject* fromlist
+#if VERYCONVOLUTED_IMPORT_NAME
+				   , PyObject* fitharg
+#endif
+				   )
 {
 	PyObject* w;
 	PyObject* x = PyDict_GetItemString(psy_get_builtins(globals),
@@ -1982,6 +2034,16 @@ static PyObject* cimpl_import_name(PyObject* globals, PyObject* name,
 				"__import__ not found");
 		return NULL;
 	}
+#if VERYCONVOLUTED_IMPORT_NAME
+	if (PyInt_AsLong(fitharg) != -1 || PyErr_Occurred())
+		w = PyTuple_Pack(5,
+				 name,
+				 globals,
+				 Py_None,
+				 fromlist,
+				 fitharg);
+	else
+#endif
 	w = Py_BuildValue("(OOOO)",
 			  name,
 			  globals,
@@ -2121,6 +2183,7 @@ static code_t* exit_function(PsycoObject* po)
 		if (!compute_and_raise_exception(po))
 			return NULL;
 		clear_pseudo_exception(po);
+                sk_incref(&psyco_skZero);
 		retsource = CompileTime_NewSk(&psyco_skZero); /*to return NULL */
 	}
 	
@@ -2980,13 +3043,24 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
 	case IMPORT_NAME:
 	{
 		PyObject* w = GETNAMEV(oparg);
-
+#if !VERYCONVOLUTED_IMPORT_NAME   /* Python < 2.5 */
 		x = psyco_generic_call(po, cimpl_import_name,
 				       CfReturnRef|CfPyErrIfNull,
 				       "vlv", LOC_GLOBALS, w, TOP());
 		if (x == NULL)
 			break;
 		POP_DECREF();
+#else
+		v = TOP();
+		u = NTOP(2);
+		x = psyco_generic_call(po, cimpl_import_name,
+				       CfReturnRef|CfPyErrIfNull,
+				       "vlvv", LOC_GLOBALS, w, v, u);
+		if (x == NULL)
+			break;
+		POP_DECREF();
+		POP_DECREF();
+#endif
 		PUSH(x);
 		goto fine;
 	}
@@ -3311,7 +3385,7 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
           if (promotion->fs == NULL)
 #endif
             code1 = psyco_finish_promotion(po, promote_me,
-                                           promotion->kflags);
+                                           promotion->pflags);
 #if USE_RUNTIME_SWITCHES
           else
             code1 = psyco_finish_fixed_switch(po, promote_me,
@@ -3387,6 +3461,7 @@ code_t* psyco_pycompiler_mainloop(PsycoObject* po)
               LOC_INLINING = po->pr.val;
               po->pr.val = NULL;
               clear_pseudo_exception(po);
+              need_reference(po, LOC_INLINING); /* see test5.makeSelection() */
               continue;
             }
 
