@@ -73,7 +73,7 @@ static bool generic_computed_cc(PsycoObject* po, vinfo_t* v)
 	/* also upon forking, because the condition codes cannot be
 	   sent as arguments (function calls typically require adds
 	   and subs to adjust the stack). */
-	extra_assert(po->ccreg == v);
+	/* 'v' should be one of the po->ccregs[_] here */
 	BEGIN_CODE
 	code = psyco_compute_cc(po, code
 #if REG_TOTAL > 0
@@ -91,26 +91,32 @@ vinfo_t* psyco_vinfo_condition(PsycoObject* po, condition_code_t cc)
   vinfo_t* result;
   if ((int)cc < CC_TOTAL)
     {
-      if (po->ccreg != NULL)
+      int index = INDEX_CC(cc);
+      if (po->ccregs[index] != NULL)
         {
           /* there is already a value in the processor flags register */
-          extra_assert(psyco_vsource_cc(po->ccreg->source) != CC_ALWAYS_FALSE);
+          condition_code_t prevcc = psyco_vsource_cc(po->ccregs[index]->source);
+          extra_assert(prevcc != CC_ALWAYS_FALSE);
           
-          if (psyco_vsource_cc(po->ccreg->source) == cc)
+          if (prevcc == cc)
             {
               /* it is the same condition, so reuse it */
-              result = po->ccreg;
+              result = po->ccregs[index];
               vinfo_incref(result);
               return result;
             }
-          /* it is not the same condition, save it */
-          BEGIN_CODE
-          NEED_CC();
-          END_CODE
+          else
+            {
+              /* it is not the same condition at all, save the old one and
+                 make a new one (should never occur with the ivm backend) */
+              BEGIN_CODE
+              NEED_CC();
+              END_CODE
+            }
         }
-      extra_assert(po->ccreg == NULL);
-      po->ccreg = vinfo_new(VirtualTime_New(cc_functions_table+(int)cc));
-      result = po->ccreg;
+      extra_assert(po->ccregs[index] == NULL);
+      result = vinfo_new(VirtualTime_New(cc_functions_table+(int)cc));
+      po->ccregs[index] = result;
     }
   else
     result = vinfo_new(CompileTime_New(cc == CC_ALWAYS_TRUE));
@@ -138,6 +144,34 @@ condition_code_t psyco_vsource_cc(Source source)
   return CC_ALWAYS_FALSE;
 }
 
+DEFINEFN
+void psyco_resolved_cc(PsycoObject* po, condition_code_t cc_known_true)
+{
+  if ((int)cc_known_true < CC_TOTAL)
+    {
+      vinfo_t* v;
+      int index;
+
+      index = INDEX_CC(cc_known_true);
+      v = po->ccregs[index];
+      if (v != NULL && psyco_vsource_cc(v->source) == cc_known_true)
+        {
+          sk_incref(&psyco_skOne);
+          v->source = CompileTime_NewSk(&psyco_skOne);
+          po->ccregs[index] = NULL;
+        }
+
+      index = INDEX_CC(INVERT_CC(cc_known_true));
+      v = po->ccregs[index];
+      if (v != NULL && psyco_vsource_cc(v->source) == INVERT_CC(cc_known_true))
+        {
+          sk_incref(&psyco_skZero);
+          v->source = CompileTime_NewSk(&psyco_skZero);
+          po->ccregs[index] = NULL;
+        }
+    }
+}
+
 #else /* if !HAVE_CCREG */
 
 # error "this was meant for ivm, but now ivm also defines HAVE_CCREG."
@@ -155,8 +189,10 @@ static bool computed_promotion(PsycoObject* po, vinfo_t* v)
   return psyco_vsource_not_important.compute_fn(po, v);
 }
 
-DEFINEVAR struct c_promotion_s psyco_nonfixed_promotion;
-DEFINEVAR struct c_promotion_s psyco_nonfixed_pyobj_promotion;
+DEFINEVAR c_promotion_t psyco_nonfixed_promotion;
+DEFINEVAR c_promotion_t psyco_nonfixed_pyobj_promotion;
+/*DEFINEVAR c_promotion_t psyco_nonfixed_promotion_mega;*/
+DEFINEVAR c_promotion_t psyco_nonfixed_pyobj_promotion_mega;
 
 DEFINEFN
 bool psyco_vsource_is_promotion(VirtualTimeSource source)
@@ -1142,14 +1178,16 @@ void psyco_codegen_init(void)
       INIT_SVIRTUAL_NOCALL(cc_functions_table[i], generic_computed_cc, 0);
     }
 #endif
+
   psyco_nonfixed_promotion.header.compute_fn = &computed_promotion;
-#if USE_RUNTIME_SWITCHES
-  psyco_nonfixed_promotion.fs = NULL;
-#endif
-  psyco_nonfixed_promotion.kflags = SkFlagFixed;
+  psyco_nonfixed_promotion.pflags = 0;
+
   psyco_nonfixed_pyobj_promotion.header.compute_fn = &computed_promotion;
-#if USE_RUNTIME_SWITCHES
-  psyco_nonfixed_pyobj_promotion.fs = NULL;
-#endif
-  psyco_nonfixed_pyobj_promotion.kflags = SkFlagFixed | SkFlagPyObj;
+  psyco_nonfixed_pyobj_promotion.pflags = PFlagPyObj;
+
+/*psyco_nonfixed_promotion_mega.header.compute_fn = &computed_promotion;
+  psyco_nonfixed_promotion_mega.pflags = PFlagMegamorphic;*/
+
+  psyco_nonfixed_pyobj_promotion_mega.header.compute_fn = &computed_promotion;
+  psyco_nonfixed_pyobj_promotion_mega.pflags = PFlagPyObj | PFlagMegamorphic;
 }
