@@ -18,32 +18,32 @@
    See the "!VLOCALS_OPC" version of compatible_array().
 */
 
-inline void fz_build(FrozenPsycoObject* fpo, vinfo_array_t* aa) {
+PSY_INLINE void fz_build(FrozenPsycoObject* fpo, vinfo_array_t* aa) {
   fpo->fz_vlocals = array_new(aa->count);
   duplicate_array(fpo->fz_vlocals, aa);
 }
 
-inline void fz_unfreeze(vinfo_array_t* aa, FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_unfreeze(vinfo_array_t* aa, FrozenPsycoObject* fpo) {
   assert_cleared_tmp_marks(fpo->fz_vlocals);
   duplicate_array(aa, fpo->fz_vlocals);
   clear_tmp_marks(fpo->fz_vlocals);
 }
 
-inline void fz_release(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_release(FrozenPsycoObject* fpo) {
   array_delete(fpo->fz_vlocals, NULL);
 }
 
       /* invariant: all snapshot.fz_vlocals in the fatlist have
          all their 'tmp' fields set to NULL. */
-inline void fz_check_invariant(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_check_invariant(FrozenPsycoObject* fpo) {
   assert_cleared_tmp_marks(fpo->fz_vlocals);
 }
 
-inline void fz_restore_invariant(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_restore_invariant(FrozenPsycoObject* fpo) {
   clear_tmp_marks(fpo->fz_vlocals);
 }
 
-inline int fz_top_array_count(FrozenPsycoObject* fpo) {
+PSY_INLINE int fz_top_array_count(FrozenPsycoObject* fpo) {
   return fpo->fz_vlocals->count;
 }
 
@@ -160,14 +160,14 @@ static void fz_internal_expand(void)
 /* Note: the pseudo-code is built in left-to-right array order,
    but it is intended to be read backwards, thus uncompression creates
    the arrays right-to-left. */
-inline void fz_putarg(Source arg)
+PSY_INLINE void fz_putarg(Source arg)
 {
   if (cmpinternal.buf_opc < (signed char*) (cmpinternal.buf_args+1))
     fz_internal_expand();
   *cmpinternal.buf_args++ = arg;
 }
 
-inline void fz_putopc(int opc)
+PSY_INLINE void fz_putopc(int opc)
 {
   if (!(-128 <= opc && opc < 128))
     {
@@ -260,12 +260,12 @@ static void fz_compress(vinfo_array_t* aa)
    the 'source' field *before* its sub-array. This will complicate
    compatible_array() a bit. */
 
-inline Source fz_getarg(void)
+PSY_INLINE Source fz_getarg(void)
 {
   return *--cmpinternal.buf_args;
 }
 
-inline int fz_getopc(void)
+PSY_INLINE int fz_getopc(void)
 {
   int result = *cmpinternal.buf_opc++;
   if (result == FZ_OPC_EXT)
@@ -376,7 +376,7 @@ static void fz_find_rt1(vinfo_array_t* aa, int length,
     }
 }
 
-inline void fz_load_fpo(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_load_fpo(FrozenPsycoObject* fpo) {
   cmpinternal.buf_opc  = (signed char*) fpo->fz_vlocals_opc;
   cmpinternal.buf_args =                fpo->fz_vlocals_opc;
 }
@@ -388,7 +388,7 @@ DEFINEFN void fz_find_runtimes(vinfo_array_t* aa, FrozenPsycoObject* fpo,
   fz_find_rt1(aa, fz_getopc(), callback, extra);
 }
 
-inline void fz_build(FrozenPsycoObject* fpo, vinfo_array_t* aa)
+PSY_INLINE void fz_build(FrozenPsycoObject* fpo, vinfo_array_t* aa)
 {
   int opc_size, arg_size;
   signed char* nbuf;
@@ -409,14 +409,14 @@ inline void fz_build(FrozenPsycoObject* fpo, vinfo_array_t* aa)
   fpo->fz_vlocals_opc = (Source*) (nbuf + arg_size);
 }
 
-inline void fz_load_fpo_stack(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_load_fpo_stack(FrozenPsycoObject* fpo) {
   fz_load_fpo(fpo);
   cmpinternal.tmp_counter = 0;
   cmpinternal.vcilink = &cmpinternal.sentinel;
   cmpinternal.sentinel.time = INT_MAX;   /* sentinel */
 }
 
-inline int fz_top_array_count(FrozenPsycoObject* fpo) {
+PSY_INLINE int fz_top_array_count(FrozenPsycoObject* fpo) {
   int result;
   if (fpo->fz_vlocals_opc == NULL)
     return 0;
@@ -426,7 +426,7 @@ inline int fz_top_array_count(FrozenPsycoObject* fpo) {
   return result;
 }
 
-inline void fz_unfreeze(vinfo_array_t* aa, FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_unfreeze(vinfo_array_t* aa, FrozenPsycoObject* fpo) {
   fz_load_fpo_stack(fpo);
   aa->count = fz_getopc();
   fz_uncompress(aa);
@@ -434,7 +434,7 @@ inline void fz_unfreeze(vinfo_array_t* aa, FrozenPsycoObject* fpo) {
   extra_assert(cmpinternal.vcilink == &cmpinternal.sentinel);
 }
 
-inline void fz_release(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_release(FrozenPsycoObject* fpo) {
   if (fpo->fz_vlocals_opc != NULL)
     {
       fz_load_fpo(fpo);
@@ -443,10 +443,10 @@ inline void fz_release(FrozenPsycoObject* fpo) {
     }
 }
 
-inline void fz_check_invariant(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_check_invariant(FrozenPsycoObject* fpo) {
 }
 
-inline void fz_restore_invariant(FrozenPsycoObject* fpo) {
+PSY_INLINE void fz_restore_invariant(FrozenPsycoObject* fpo) {
 }
 
 
@@ -482,6 +482,9 @@ void fpo_release(FrozenPsycoObject* fpo)
 
 static void fpo_find_regs_array(vinfo_array_t* source, PsycoObject* po)
 {
+#if HAVE_CCREG
+  condition_code_t cc;
+#endif
   int i = source->count;
   while (i--)
     {
@@ -495,8 +498,8 @@ static void fpo_find_regs_array(vinfo_array_t* source, PsycoObject* po)
           else
 #endif
 #if HAVE_CCREG
-            if (psyco_vsource_cc(src) != CC_ALWAYS_FALSE)
-              po->ccreg = a;
+            if ((cc = psyco_vsource_cc(src)) != CC_ALWAYS_FALSE)
+              po->ccregs[INDEX_CC(cc)] = a;
             else
 #endif
               /* nothing */ ;
@@ -688,6 +691,7 @@ void* psyco_prepare_respawn_ex(PsycoObject* po, condition_code_t jmpcondition,
       /* write the jump to the proxy */
       po->code = calling_code;
       po->codelimit = calling_limit;
+      psyco_resolved_cc(po, INVERT_CC(jmpcondition)); /* no jump => cond false */
       rs->write_jmp = conditional_jump_to(po, (code_t*)codebuf->codestart,
                                           jmpcondition);
       dump_code_buffers();
@@ -991,7 +995,7 @@ static bool compatible_array(vinfo_array_t* aa, vinfo_array_t* bb,
   return false;
 }
 
-inline bool fz_compatible_array(vinfo_array_t* aa, FrozenPsycoObject* fpo,
+PSY_INLINE bool fz_compatible_array(vinfo_array_t* aa, FrozenPsycoObject* fpo,
                                 vcompatible_t* result) {
   return compatible_array(aa, fpo->fz_vlocals, &result->diff);
 }
@@ -1038,7 +1042,7 @@ static bool compatible_vinfo(vinfo_t* a, Source bsource, int bcount,
       switch (gettime(a->source)) {
         
       case RunTime:
-        if ((diff & (RunTime_NoRef|RunTime_NonNeg)) != 0)
+        if ((diff & (RunTime_NoRef|RunTime_NonNeg|RunTime_Megamorphic)) != 0)
           {
             /* from 'with ref' to 'without ref' or vice-versa:
                a source in 'a' with reference cannot pass for
@@ -1052,6 +1056,11 @@ static bool compatible_vinfo(vinfo_t* a, Source bsource, int bcount,
             if ((a->source & RunTime_NonNeg) == 0 &&
                 (  bsource & RunTime_NonNeg) != 0)
               return false;
+            /* a source in 'a' which is flagged as megamorphic should
+               not fall back to a source in 'b' which is not */
+            if ((a->source & RunTime_Megamorphic) != 0 &&
+                (  bsource & RunTime_Megamorphic) == 0)
+              return false;
           }
         break;
 
@@ -1344,7 +1353,7 @@ static void skip_compatible_array(int count)
 }
 #endif  /* COMPRESS_COMPILETIME_SUBITEMS */
 
-inline bool fz_compatible_array(vinfo_array_t* aa, FrozenPsycoObject* fpo,
+PSY_INLINE bool fz_compatible_array(vinfo_array_t* aa, FrozenPsycoObject* fpo,
                                 vcompatible_t* result) {
   bool ok;
   fz_load_fpo_stack(fpo);
@@ -1657,7 +1666,7 @@ typedef struct { /* produced at compile time and read by the dispatcher */
 
 #if PROMOTION_TACTIC == 0
 #define NEED_PYOBJ_KEY
-inline code_t* lookup_old_promotion_values(rt_promotion_t* fs,
+PSY_INLINE code_t* lookup_old_promotion_values(rt_promotion_t* fs,
                                            PyObject* key)
 {
   /* have we already seen this value? */
@@ -1673,7 +1682,7 @@ inline code_t* lookup_old_promotion_values(rt_promotion_t* fs,
 #  if PROMOTION_FAST_COMMON_CASE
 static
 #  else
-inline
+PSY_INLINE
 #  endif
 code_t* lookup_old_promotion_values(rt_promotion_t* fs, long value)
 {
@@ -1735,6 +1744,19 @@ static int quick_lookup_counter = 0;
       } while (0)
 #endif
 
+PSY_INLINE int lookup_count_previous_values(rt_promotion_t* fs,
+					    rt_local_buf_t** megabuf)
+{
+	int result = 0;
+	rt_local_buf_t* buf = fs->local_chained_list;
+	while (buf) {
+		if (buf->key == -1)   /* use (PyObject*)(-1) as a marker */
+			*megabuf = buf;
+		result++;
+		buf = buf->next;
+	}
+	return result;
+}
 #endif  /* PROMOTION_TACTIC == 1 */
 
 
@@ -1766,6 +1788,12 @@ static code_t* do_promotion_internal(rt_promotion_t* fs,
   
   /* fix the value of 'v' */
   CHKTIME(v->source, RunTime);   /* from run-time to compile-time */
+  if (!RSOURCE_REG_IS_NONE(v->source))
+    {
+      /* remove this value from 'po->regarray' */
+      REG_NUMBER(po, RSOURCE_REG(v->source)) = NULL;
+      SET_RUNTIME_REG_TO_NONE(v);
+    }
   v->source = CompileTime_NewSk(sk);
   /* compile from this new state, in which 'v' has been promoted to
      compile-time. */
@@ -1811,6 +1839,65 @@ static code_t* do_promotion_internal(rt_promotion_t* fs,
   return result;
 }
 
+static code_t* detected_megamorphic_pyobj_site(rt_promotion_t* fs)
+{
+  CodeBufferObject* codebuf;
+  code_t* result;
+  vinfo_t* v;
+  PsycoObject* newpo;
+  PsycoObject* po = fs->po;
+  mergepoint_t* mp;
+
+  /* get a copy of the compiler state */
+  newpo = PsycoObject_Duplicate(po);
+  if (newpo == NULL)
+    OUT_OF_MEMORY();
+  /* store the copy back into 'fs' and use the old 'po' to compile.
+     We do so because in 'newpo' all 'tmp' fields are now NULL,
+     but no longer in 'po'. */
+  fs->po = newpo;
+  v = fs->fix;      /* get the variable we will mark as megamorphic... */
+  fs->fix = v->tmp; /*     ...and update 'fs' with its copy in 'newpo'     */
+  
+  /* mark 'v' as megamorphic */
+  CHKTIME(v->source, RunTime);
+  v->source |= RunTime_Megamorphic;
+  /* compile from this new state, in which 'v' has been flagged */
+  mp = psyco_exact_merge_point(po->pr.merge_points, po->pr.next_instr);
+
+#if PROMOTION_TACTIC == 0
+#  error "XXX reimplement"
+#endif
+
+#if PROMOTION_TACTIC == 1
+  codebuf = psyco_new_code_buffer(NULL, NULL, &po->codelimit);
+  {
+    code_t* codeend;
+    rt_local_buf_t* buf = (rt_local_buf_t*) codebuf->codestart;
+    code_t* code = (code_t*)(buf+1);
+    ALIGN_NO_FILL();
+    result = code;
+    buf = ((rt_local_buf_t*) code) - 1;
+    
+# if CODE_DUMP
+    memset(codebuf->codestart, 0xCC, ((char*) buf) - ((char*) codebuf->codestart));
+    buf->signature = 0x66666666;
+# endif
+    buf->next = fs->local_chained_list;
+    buf->key = -1;   /* use (PyObject*)(-1) as a marker */
+    fs->local_chained_list = buf;
+    
+    po->code = insn_code_label(result);
+    codeend = psyco_compile(po, mp, false);
+    psyco_shrink_code_buffer(codebuf, codeend);
+    /* XXX don't know what to do with reference to 'codebuf' */
+  }
+#endif
+
+  dump_code_buffers();
+  return result;
+}
+
 
 /* NOTE: the following two functions must be as fast as possible, because
    they are called from the run-time code even during normal (non-compiling)
@@ -1865,8 +1952,41 @@ static code_t* do_promotion_pyobj(rt_promotion_t* fs, PyObject* key)
   return fix_fast_common_case(fs, (long) key, result);
 }
 
+static code_t* do_promotion_pyobj_mega(rt_promotion_t* fs, PyObject* key)
+{
+  code_t* result;
+
+#ifdef NEED_PYOBJ_KEY
+  PyObject* key1 = key;
+#else
+  long key1 = (long) key;
+#endif
+
+  /* have we already seen this value? */
+  QUICK_LOOKUP_PROMOTION_VALUE(fs, key1, result);
+  if (result == NULL)
+    {
+      /* no -> did we already promote many objects here? */
+      rt_local_buf_t* megabuf = NULL;
+      if (lookup_count_previous_values(fs, &megabuf) >= MEGAMORPHIC_MAX)
+        {
+          /* yes -> stop specializing */
+          if (megabuf == NULL)
+            return detected_megamorphic_pyobj_site(fs);
+          else
+            return (code_t*)(megabuf+1);  /* mega version already compiled */
+        }
+      /* no -> we must build new code */
+      Py_INCREF(key);
+      result = do_promotion_internal(fs, key1, sk_new((long) key,
+                                                      SkFlagFixed|SkFlagPyObj));
+    }
+  /* done -> jump to the codebuf */
+  return fix_fast_common_case(fs, (long) key, result);
+}
+
 DEFINEFN
-code_t* psyco_finish_promotion(PsycoObject* po, vinfo_t* fix, long kflags)
+code_t* psyco_finish_promotion(PsycoObject* po, vinfo_t* fix, int pflags)
 {
   rt_promotion_t* fs;
   void* do_promotion;
@@ -1878,10 +1998,21 @@ code_t* psyco_finish_promotion(PsycoObject* po, vinfo_t* fix, long kflags)
   TRACE_EXECUTION("PROMOTION");
 
   /* write the code that calls the proxy 'do_promotion' */
-  if ((kflags & SkFlagPyObj) == 0)
+  switch (pflags) {
+  case 0:
     do_promotion = &do_promotion_long;
-  else
+    break;
+  case PFlagPyObj:
     do_promotion = &do_promotion_pyobj;
+    break;
+  /*case PFlagMegamorphic: ... */
+  case PFlagPyObj | PFlagMegamorphic:
+    do_promotion = &do_promotion_pyobj_mega;
+    break;
+  default:
+    psyco_fatal_msg("bad pflags");
+    return NULL;
+  }
   fs = (rt_promotion_t*) ipromotion_finish(po, fix, do_promotion);
 
   /* fill in the constant structure that 'do_promotion' will get as parameter */
