@@ -7,6 +7,7 @@ explicitnumber = {
     '0x010300f0': '1.3',
     '0x010400f0': '1.4',
     '0x010500f0': '1.5',
+    '0x010501f0': '1.5.1',
     }
 
 FILES = {
@@ -14,7 +15,7 @@ FILES = {
     '../py-support/support.py': re.compile(r'__version__\s*=\s*(0x[0-9a-fA-F]+)'),
     '../doc/psycoguide.tex': re.compile(r'\\release\{([0-9.]+)\}'),
     '../setup.py': re.compile(r'version\s*=\s*\"([0-9.]+)\"'),
-#    '../../my/psyco/dist/Makefile': re.compile(r'version\s*=\s*([0-9.]+)'),
+    '../../www/dist/Makefile': re.compile(r'version\s*=\s*([0-9.]+)'),
     '../../www/content/index.rst': re.compile(r'`Psyco ([0-9.]+)@'),
     '../../www/content/download.rst': re.compile(r'Current version is ([0-9.]+)'),
     '../../www/content/./download.rst': re.compile(r'[dD]ownload Release ([0-9.]+)'),
@@ -40,3 +41,5 @@ if len(versions) != 1:
     raise Exception, versions
 else:
     print "versionchecker: ok"
+
+import files; files.main()
