@@ -1,4 +1,4 @@
-import psyco, dis, types, sys
+import psyco, dis, types, sys, math
 
 def f(filename, mode='r'):
     s = ""
@@ -45,6 +45,7 @@ def booltest():
     print [a & b for a in (False,True) for b in (False,True)]
     print [a | b for a in (False,True) for b in (False,True)]
     print [a ^ b for a in (False,True) for b in (False,True)]
+    print not (False in ())
 
 def exc_test():
     try:
@@ -95,7 +96,7 @@ def f6(n):
         print g6(x=n)
         print g6bis(n)
     except:
-        print sys.exc_type
+        print sys.exc_type.__name__
 
 def g7(x, *args):
     if not g7len(args):
@@ -307,13 +308,118 @@ else:
         class S(array.array): pass
         print type(S('i')).__name__
 
+
+class CompositeElement:
+    def __init__(self):
+        self.children = []
+
+    def append(self, child):
+        self.children.append(child)
+        return self
+
+def makeSelection():
+    """Segfault bug (thanks xorAxAx)"""
+    result = CompositeElement()
+    result.append(CompositeElement().append(42))
+    print 'do stuff here'
+    assert isinstance(result.children[0], CompositeElement)
+
+class Class1:
+    __metaclass__ = type
+def class_creation_1(n=400000):
+    for i in xrange(n):
+        Class1()
+    print "ok"
+
+class Class2:
+    __metaclass__ = type
+    def __init__(self, a):
+        self.a = a
+def class_creation_2(n=400000):
+    for i in xrange(n):
+        c = Class2(i)
+        assert c.a == i
+    print "ok"
+
+class Class3:
+    __metaclass__ = type
+    def __init__(self):
+        return 42
+def class_creation_3():
+    try:
+        Class3()
+    except TypeError:
+        if (2,2) <= sys.version_info < (2,5):
+            print "got a TypeError, but new-style classes don't"
+            print "check the __init__() return value before 2.5"
+        else:
+            print "ok"
+    else:
+        if (2,2) <= sys.version_info < (2,5):
+            print "ok"
+        else:
+            print "__init__() => 42: should have raised TypeError!"
+
+class Class4:
+    __metaclass__ = type
+    def __init__(self, a=None):
+        self.a = a
+def class_creation_4(n=200000):
+    for i in xrange(n):
+        c = Class4(a=i)
+        assert c.a == i
+    print "ok"
+
+def power_int(n=2000):
+    for j in range(n):
+        x = 0
+        for i in range(1000):
+            x += i ** 2
+    print x
+
+def power_int_long(n=200):
+    for j in range(n):
+        x = 0
+        for i in range(1000):
+            x += i ** 2L
+    print type(x).__name__[:4], int(x)
+
+def power_float(n=500):
+    for j in range(n):
+        x = 0.0
+        for i in range(1000):
+            x += i ** 1.409999992
+    print type(x).__name__, int(x)
+
+def conditional_doubletest_fold():
+    n = sys.getrefcount(conditional_doubletest_fold)   # anything > 0
+    if n < 0 and expression_not_evaluated:
+        print "BUG"
+    else:
+        print "ok(1)"
+    n = sys.getrefcount(conditional_doubletest_fold)   # anything > 0
+    if n >= 0 or expression_not_evaluated:
+        print "ok(2)"
+    else:
+        print "BUG"
+
+def importname(value):
+    for v in value:
+        import sys
+    print "Ok"
+
 if __name__ == '__main__':
+    from test1 import go, print_results
     import time
     print "break!"
     time.sleep(0.5)
-    #psyco.full()
-    #longrangetest()
-    #psyco.proxy(arraytest)()
-    #proxy_defargs()
-    setfilter()
+    #go(class_creation_1)
+    #go(class_creation_2)
+    #go(class_creation_3)
+    #go(class_creation_4)
+    #go(power_int)
+    #go(power_int_long)
+    #go(power_float)
+    psyco.proxy(conditional_doubletest_fold)()
     psyco.dumpcodebuf()
+    print_results()
